"""Asynchronous Python client for Cybro."""
from .cybro import Cybro
from .cybro import CybroConnectionError
from .cybro import CybroConnectionTimeoutError
from .cybro import CybroError
from .models import Device
from .models import ServerInfo
from .models import Var
from .models import VarType

__all__ = [
    "Device",
    "ServerInfo",
    "VarType",
    "Var",
    "Cybro",
    "CybroConnectionError",
    "CybroConnectionTimeoutError",
    "CybroError",
]



"""Asynchronous Python client for Cybro scgi server."""
from __future__ import annotations

import asyncio
import json
import socket
from dataclasses import dataclass
from itertools import islice
from typing import Any

import aiohttp
import async_timeout
import backoff
import xmltodict
from cachetools import TTLCache
from yarl import URL

from .exceptions import CybroConnectionError
from .exceptions import CybroConnectionTimeoutError
from .exceptions import CybroEmptyResponseError
from .exceptions import CybroError
from .models import Device
from .models import VarType

VERSION_CACHE: TTLCache = TTLCache(maxsize=16, ttl=7200)

VAR_CHUNK_SIZE: int = 25


@dataclass
class Cybro:
    """Main class for handling connections with Cybro scgi server."""

    host: str
    port: int = 4000
    nad: int = 0
    path: str = ""
    request_timeout: float = 8.0
    session: aiohttp.client.ClientSession | None = None

    _device: Device | None = None

    def __init__(
        self,
        host_str: str,
        port: int = 4000,
        nad: int = 0,
        session: aiohttp.client.ClientSession | None = None,
    ) -> None:
        """Define a new Cybro scgi server session.

        Args:
            host_str: Cybro scgi server connection string
            port: Cybro scgi server port (Default: 4000)
            nad: Cybro PLC NAD (Network address)
            session: optional a aiohttp session
        """
        new_host = host_str
        new_path = ""
        if new_host.find("//") >= 0:
            new_host = new_host.split("//")[1]
        if new_host.find("/") >= 0:
            new_path = "/" + "/".join(new_host.split("/")[1:])
            new_host = new_host.split("/")[0]
        url = URL.build(scheme="http", host=new_host, path=new_path)
        self.host = url.host
        self.path = url.path
        self.port = port
        self.nad = nad
        if session is not None:
            self.session = session

    async def disconnect(self) -> None:
        """Disconnect from cybro scgi server object."""
        if self.session is not None:
            await self.session.close()
            self.session = None

    @backoff.on_exception(
        backoff.expo,
        {CybroConnectionError, CybroConnectionTimeoutError, CybroError},
        max_tries=3,
        logger=None,
    )
    async def request(
        self,
        data: dict | str | None = None,
    ) -> Any:
        """Handle a request to a scgi server.

        A generic method for sending/handling HTTP requests done gainst
        the scgi server.

        Args:
            data: string / Dictionary of data to send to the scgi server.

        Returns:
            A Python dictionary with the response from the scgi server.

        Raises:
            CybroConnectionError: An error occurred while communicating with
                the scgi server.
            CybroConnectionTimeoutError: A timeout occurred while communicating
                with the scgi server.
            CybroError: Received an unexpected response from Cybro scgi server.
        """
        if isinstance(data, str):
            url = URL.build(
                scheme="http",
                host=self.host,
                port=self.port,
                path=self.path,
                query_string=data,
            )
        else:
            url = URL.build(
                scheme="http",
                host=self.host,
                port=self.port,
                path=self.path,
                query=data,
            )

        # some fix of query data
        url_fixed = str(url).replace("=&", "&").removesuffix("=")
        url = url_fixed

        headers = {
            "Accept": "text/plain, */*",
        }

        if self.session is None:
            self.session = aiohttp.client.ClientSession()

        try:
            async with async_timeout.timeout(self.request_timeout):
                response = await self.session.get(
                    url=url_fixed,
                    allow_redirects=False,
                    ssl=False,
                    headers=headers,
                )

            content_type = response.headers.get("Content-Type", "")

            if response.status // 100 in [4, 5]:
                contents = await response.read()
                response.close()

                if content_type == "application/json":
                    raise CybroError(
                        response.status, json.loads(contents.decode("utf8"))
                    )
                raise CybroError(response.status, {"message": contents.decode("utf8")})

            response_data = xmltodict.parse(await response.text())

        except asyncio.TimeoutError as exception:
            raise CybroConnectionTimeoutError(
                f"Timeout occurred while connecting to server at {self.host}:{self.port}"
            ) from exception
        except (aiohttp.ClientError, socket.gaierror) as exception:
            print(exception)
            raise CybroConnectionError(
                f"Error occurred while communicating with server at {self.host}:{self.port}"
            ) from exception

        return response_data.get("data")

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def update(
        self, full_update: bool = False, plc_nad: int = 0, device_type: int = 0
    ) -> Device:
        """Get all variables in a single call.

        This method updates all variable information with a single call.

        Args:
            full_update: Force a full update from the device Device.
            plc_nad: Address of PLC to read
            device_type: 0 = undefined / generic PLC, 1 = HIQ-controller

        Returns:
            Cybro Device data.

        Raises:
            CybroEmptyResponseError: The Cybro scgi server returned an empty response.
        """
        if self._device is None or full_update:
            # read all relevant server vars
            _vars: dict[str, str] = {
                "sys.scgi_port_status": "",
                "sys.server_uptime": "",
                "sys.scgi_request_pending": "",
                "sys.scgi_request_count": "",
                "sys.push_port_status": "",
                "sys.push_count": "",
                "sys.push_ack_errors": "",
                "sys.push_list_count": "",
                "sys.cache_request": "",
                "sys.cache_valid": "",
                "sys.server_version": "",
                "sys.udp_rx_count": "",
                "sys.udp_tx_count": "",
                "sys.datalogger_status": "",
            }
            if plc_nad != 0 and self.nad == 0:
                self.nad = plc_nad
            if self.nad != 0:
                # read also specific plc variables
                _controller = "c" + str(self.nad) + "."
                _vars[_controller + "sys.ip_port"] = ""
                _vars[_controller + "sys.timestamp"] = ""
                _vars[_controller + "sys.plc_program_status"] = ""
                _vars[_controller + "sys.response_time"] = ""
                _vars[_controller + "sys.bytes_transferred"] = ""
                _vars[_controller + "sys.comm_error_count"] = ""
                _vars[_controller + "sys.alc_file"] = ""
                # read / prepare specific vars for HIQ-controller
                if device_type == 1:
                    _vars = _add_hiq_tags(_vars, _controller)

            if not (data := await self.request(data=_vars)):
                raise CybroEmptyResponseError(
                    f"Cybro scgi server at {self.host}:{self.port} returned an empty API"
                    " response on full update"
                )

            self._device = Device(data, plc_nad=self.nad)

            if len(self._device.user_vars) > 0:
                _user_vars = _get_chunk(self._device.user_vars, VAR_CHUNK_SIZE)
                for _vars in _user_vars:
                    if not (data := await self.request(data=_vars)):
                        raise CybroEmptyResponseError(
                            f"Cybro scgi server at {self.host}:{self.port} returned an empty"
                            " response on full update"
                        )
                    self._device.update_user_var_from_dict(data=data)

            return self._device

        if len(self._device.user_vars) > 0:
            _user_vars = _get_chunk(self._device.user_vars, VAR_CHUNK_SIZE)
            for _vars in _user_vars:
                if not (data := await self.request(data=_vars)):
                    raise CybroEmptyResponseError(
                        f"Cybro scgi server at {self.host}:{self.port} returned an empty"
                        " response on user update"
                    )
                self._device.update_user_var_from_dict(data=data)

            return self._device

        return self._device

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def write_var(
        self, name: str, value: str, var_type: VarType = VarType.STR
    ) -> str | int | float | bool:
        """Write a single variable to scgi server."""
        data = await self.request(data={name: value})
        return self._device.update_var(data, var_type=var_type)

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def read_var(
        self, name: str, var_type: VarType = VarType.STR
    ) -> str | int | float | bool:
        """Read a single variable from scgi server."""
        if not (data := await self.request(data=name)):
            raise CybroEmptyResponseError(
                f"Cybro scgi server at {self.host}:{self.port} returned an empty"
                " response on read of {name}"
            )
        return self._device.update_var(data, var_type=var_type)

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def read_var_int(
        self,
        name: str,
    ) -> int:
        """Read a single variable from scgi server as int."""
        return await self.read_var(name, VarType.INT)

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def read_var_float(
        self,
        name: str,
    ) -> float:
        """Read a single variable from scgi server as float."""
        return await self.read_var(name, VarType.FLOAT)

    @backoff.on_exception(
        backoff.expo, CybroEmptyResponseError, max_tries=3, logger=None
    )
    async def read_var_bool(
        self,
        name: str,
    ) -> bool:
        """Read a single variable from scgi server as float."""
        return await self.read_var(name, VarType.BOOL)

    def add_var(self, name: str) -> None:
        """Add a variable into update buffer."""
        self._device.add_var(name)

    def remove_var(self, name: str) -> None:
        """Remove a variable from update buffer."""
        self._device.remove_var(name)

    async def __aenter__(self) -> Cybro:
        """Async enter.

        Returns:
            The Cybro object.
        """
        return self

    async def __aexit__(self, *_exc_info) -> None:
        """Async exit.

        Args:
            _exc_info: Exec type.
        """


def _get_chunk(data, chunk_size):
    """Split dictionary into smaller chunks."""
    data_it = iter(data)
    for i in range(0, len(data), chunk_size):
        yield {k: data[k] for k in islice(data_it, chunk_size)}


def _add_hiq_tags(
    variables: dict[str, str], controller: str = "c10000."
) -> dict[str, str]:
    """Adds controller specific general error tags.

    Args:
        variables: current list of variables to add to it
        controller: controller prefix (Default: c10000.)

    Returns:
        An updated dictionary of variables that includes HIQ controller specific variables.
    """
    _vars = variables
    _vars[controller + "lc00_general_error"] = ""
    _vars[controller + "lc01_general_error"] = ""
    _vars[controller + "lc02_general_error"] = ""
    _vars[controller + "lc03_general_error"] = ""
    _vars[controller + "lc04_general_error"] = ""
    _vars[controller + "lc05_general_error"] = ""
    _vars[controller + "lc06_general_error"] = ""
    _vars[controller + "lc07_general_error"] = ""
    _vars[controller + "ld00_general_error"] = ""
    _vars[controller + "ld01_general_error"] = ""
    _vars[controller + "ld02_general_error"] = ""
    _vars[controller + "ld03_general_error"] = ""
    _vars[controller + "bc00_general_error"] = ""
    _vars[controller + "bc01_general_error"] = ""
    _vars[controller + "bc02_general_error"] = ""
    _vars[controller + "bc03_general_error"] = ""
    _vars[controller + "bc04_general_error"] = ""
    _vars[controller + "bc05_general_error"] = ""
    _vars[controller + "sc00_general_error"] = ""
    _vars[controller + "sc01_general_error"] = ""
    _vars[controller + "sc02_general_error"] = ""
    _vars[controller + "sc03_general_error"] = ""
    _vars[controller + "th00_general_error"] = ""
    _vars[controller + "th01_general_error"] = ""
    _vars[controller + "th02_general_error"] = ""
    _vars[controller + "th03_general_error"] = ""
    _vars[controller + "th04_general_error"] = ""
    _vars[controller + "th05_general_error"] = ""
    _vars[controller + "fc00_general_error"] = ""
    _vars[controller + "fc01_general_error"] = ""
    _vars[controller + "fc02_general_error"] = ""
    _vars[controller + "fc03_general_error"] = ""
    _vars[controller + "fc04_general_error"] = ""
    _vars[controller + "fc05_general_error"] = ""
    _vars[controller + "power_meter_error"] = ""
    return _vars



"""Models for Cybro scgi server objects."""
from __future__ import annotations

from dataclasses import dataclass
from enum import IntEnum
from typing import Any

from .exceptions import CybroError
from .exceptions import CybroPlcNotFoundError


@dataclass
class ServerInfo:  # pylint:
    """Cybro scgi server information."""

    scgi_port_status: str
    """- no response, server is down
    - "active", server is up and running"""
    server_uptime: str
    """server uptime, returned as "dd days, hh:mm:ss"""
    scgi_request_pending: int
    """number of requests waiting to be served"""
    scgi_request_count: int
    """total number of requests served since server is started"""
    push_port_status: str
    """- "error", port already used by another application
    - "inactive", push disabled by configuration file
    - "active", push server is up and running"""
    push_count: int
    """total number of push messages received from controllers"""
    push_ack_errors: int
    """total number of push acknowledge errors"""
    push_list_count: int
    """total number of controllers in push list"""
    cache_request: int
    """configured read request time [s]"""
    cache_valid: int
    """configured cache validity time [s]"""
    server_version: str
    """returns "major.minor.release" """
    udp_rx_count: int
    """total number of messages received through UDP proxy"""
    udp_tx_count: int
    """total number of messages transmitted through UDP proxy"""
    datalogger_status: str
    """- "active", process running
    - "stopped", process stopped"""
    nad_list: str = ""
    """list of controllers in push list"""
    push_list: str = ""
    """list of controllers with status
    (nad, ip:port, plc status, plc program, alc file, resp time)"""
    abus_list: str = ""
    """status of communication between server and controllers"""
    datalogger_list: str = ""
    """list of variables for sample and alarm"""
    proxy_activity_list: str = ""
    """data for proxy activity"""

    @staticmethod
    def from_dict(data: dict[str, Any]) -> ServerInfo:
        """generate ServerInfo object out of name / value dictionary

        Args:
            data: Received data from SCGI server.

        Returns:
            ServerInfo data.

        Raises:
            CybroError: The Cybro SCGI server returned no or incomlete Server info data.
        """
        try:
            return ServerInfo(
                scgi_port_status=data["sys.scgi_port_status"],
                server_uptime=data["sys.server_uptime"],
                scgi_request_pending=data["sys.scgi_request_pending"],
                scgi_request_count=data["sys.scgi_request_count"],
                push_port_status=data["sys.push_port_status"],
                push_count=data["sys.push_count"],
                push_ack_errors=data["sys.push_ack_errors"],
                push_list_count=data["sys.push_list_count"],
                cache_request=data["sys.cache_request"],
                cache_valid=data["sys.cache_valid"],
                server_version=data["sys.server_version"],
                udp_rx_count=data["sys.udp_rx_count"],
                udp_tx_count=data["sys.udp_tx_count"],
                datalogger_status=data["sys.datalogger_status"],
            )
        except KeyError:
            raise CybroError("Not all ServerInfo tags found in data") from KeyError

    @staticmethod
    def from_vars(variables: dict[str, Var]) -> ServerInfo:
        """generate ServerInfo object out of name / Var object dictionary

        Args:
            variables: Received data from SCGI server.

        Returns:
            ServerInfo data.

        Raises:
            CybroError: The Cybro scgi server returned no or incomlete Server info data.
        """
        try:
            return ServerInfo(
                scgi_port_status=variables.get("sys.scgi_port_status").value,
                server_uptime=variables.get("sys.server_uptime").value,
                scgi_request_pending=variables.get("sys.scgi_request_pending").value,
                scgi_request_count=variables.get("sys.scgi_request_count").value,
                push_port_status=variables.get("sys.push_port_status").value,
                push_count=variables.get("sys.push_count").value,
                push_ack_errors=variables.get("sys.push_ack_errors").value,
                push_list_count=variables.get("sys.push_list_count").value,
                cache_request=variables.get("sys.cache_request").value,
                cache_valid=variables.get("sys.cache_valid").value,
                server_version=variables.get("sys.server_version").value,
                udp_rx_count=variables.get("sys.udp_rx_count").value,
                udp_tx_count=variables.get("sys.udp_tx_count").value,
                datalogger_status=variables.get("sys.datalogger_status").value,
            )
        except AttributeError:
            raise CybroError(
                "Not all ServerInfo tags found in data"
            ) from AttributeError


@dataclass
class PlcInfo:  # pylint:
    """Cybro PLC information."""

    nad: int
    """Cybro PLC NAD"""
    ip_port: str
    """ip address and port, returned as "xxx.xxx.xxx.xxx:yyyy" """
    timestamp: str
    """time and date when the program is sent to controller"""
    plc_program_status: str
    """- "?", status unknown
    - "-", no communication, controller is offline
    - "missing", controller has no plc program
    - "ok", controller is up and running"""
    response_time: str
    """number of milliseconds elapsed between request and response"""
    bytes_transferred: int
    """number of bytes, read and write, transferred between server and controller"""
    comm_error_count: int
    """total number of communication errors encountered by controller"""
    alc_file: str
    """allocation file, directly from the Cybro file system"""
    plc_vars: dict[str, str]
    """holds all plc vars (needs to be filled after generating PLCInfo object by calling parse_alc_file())"""

    @staticmethod
    def from_dict(data: dict[str, Any], plc_nad: int) -> PlcInfo:
        """generate PlcInfo object out of name / value dictionary

        Args:
            data: Received data from SCGI server.
            plc_nad: Address of PLC

        Returns:
            PlcInfo data.

        Raises:
            CybroPlcNotFoundError: The Cybro scgi server returned no or incomlete PLC data.
        """
        try:
            return PlcInfo(
                nad=plc_nad,
                ip_port=data["c" + str(plc_nad) + ".sys.ip_port"],
                timestamp=data["c" + str(plc_nad) + ".sys.timestamp"],
                plc_program_status=data["c" + str(plc_nad) + ".sys.plc_program_status"],
                response_time=data["c" + str(plc_nad) + ".sys.response_time"],
                bytes_transferred=data["c" + str(plc_nad) + ".sys.bytes_transferred"],
                comm_error_count=data["c" + str(plc_nad) + ".sys.comm_error_count"],
                alc_file=data["c" + str(plc_nad) + ".sys.alc_file"],
                plc_vars={},
            )
        except KeyError:
            raise CybroPlcNotFoundError(
                "Not all ServerInfo tags found in data"
            ) from KeyError

    @staticmethod
    def from_vars(variables: dict[str, Var], plc_nad: int) -> PlcInfo:
        """generate PlcInfo object out of name / Var object dictionary

        Args:
            variables: Force a full update from the device Device.
            plc_nad: Address of PLC

        Returns:
            PlcInfo data.

        Raises:
            CybroPlcNotFoundError: The Cybro scgi server returned no or incomlete PLC data.
        """
        nad = plc_nad
        try:
            return PlcInfo(
                nad=nad,
                ip_port=variables.get("c" + str(plc_nad) + ".sys.ip_port", "").value,
                timestamp=variables.get(
                    "c" + str(plc_nad) + ".sys.timestamp", ""
                ).value,
                plc_program_status=variables.get(
                    "c" + str(plc_nad) + ".sys.plc_program_status", ""
                ).value,
                response_time=variables.get(
                    "c" + str(plc_nad) + ".sys.response_time", ""
                ).value,
                bytes_transferred=variables.get(
                    "c" + str(plc_nad) + ".sys.bytes_transferred", ""
                ).value,
                comm_error_count=variables.get(
                    "c" + str(plc_nad) + ".sys.comm_error_count", ""
                ).value,
                alc_file=variables.get("c" + str(plc_nad) + ".sys.alc_file", "").value,
                plc_vars={},
            )
        except AttributeError:
            raise CybroPlcNotFoundError(
                f"Cybro PLC with NAD {nad} not found"
            ) from AttributeError

    def parse_alc_file(self) -> dict[str, str]:
        """Shall be called after update of PlcInfo to refresh list of all plc vars"""
        prefix = f"c{self.nad}."
        res: dict[str, str] = {}
        if self.alc_file is None:
            self.plc_vars = res
            return res
        alc_lines = self.alc_file.splitlines()
        for line in alc_lines[2:]:
            typ = line[37:43].rstrip()
            name = line[43:-1].split()[0].rstrip()
            res[f"{prefix}{name}"] = typ
            self.plc_vars = res
        return res


@dataclass
class Var:
    """object representing a scgi server variable"""

    name: str
    """name of variable"""
    value: str
    """value as string of variable"""
    description: str
    """description of the variable"""

    def __init__(self, name: str, value: str, description: str) -> None:
        self.name = name
        self.value = value
        self.description = description

    @staticmethod
    def from_dict(data: dict[str, Any]) -> Var:
        """split a dict with "name", "value" and "description" into a Var object"""
        return Var(
            name=data.get("name"),
            value=data.get("value"),
            description=data.get("description"),
        )

    def value_string(self) -> str:
        """get current value"""
        return self.value

    def value_int(self) -> int:
        """get current value"""
        return int(self.value)

    def value_bool(self) -> bool:
        """get current value"""
        return bool(self.value == "1")

    def value_float(self) -> float:
        """get current value"""
        return float(self.value)


class Device:
    """Object holding all information of Cybro scgi server."""

    info: str
    server_info: ServerInfo
    plc_info: PlcInfo | None
    vars: dict[str, Var] = {}
    """list of variable / value / descriptions (after read/write)"""
    user_vars: dict[str, str] = {}
    """list of all variables to periodically update"""
    vars_types: dict[str, int] = {}
    """list of variable types"""

    def __init__(self, data: dict, plc_nad: int = 0) -> None:
        """Initialize an empty Cybro scgi server device class.

        Args:
            data: The full API response from a Cybro scgi server device.
            plc_nad: the Cybro PLC NAD

        Raises:
            CybroError: In case the given API response is incomplete in a way
                that a Device object cannot be constructed from it.
        """
        # Check if all elements are in the passed dict, else raise an Error
        try:
            if "var" in data:
                self.update_from_dict(data, plc_nad=plc_nad)
            else:
                raise CybroError(
                    "Cybro data is incomplete, cannot construct device object"
                )
        except TypeError as exc:
            raise CybroError(
                "Cybro data is incomplete, cannot construct device object"
            ) from exc

    def update_from_dict(self, data: dict, plc_nad: int = 0) -> Device:
        """Return Device object from Cybro scgi server response.

        Args:
            data: Update the device object with the data received from a
                Cybro scgi server request.
            plc_nad: Cybro PLC NAD

        Returns:
            The updated Device object.
        """
        # update server info
        for var in data["var"]:
            self.vars.update({var["name"]: Var.from_dict(var)})
            self.vars_types.update({var["name"]: 0})
        self.server_info = ServerInfo.from_vars(self.vars)
        self.info = "CybrotechScgiServer v" + self.server_info.server_version

        if plc_nad != 0:
            self.plc_info = PlcInfo.from_vars(self.vars, plc_nad)
            self.plc_info.parse_alc_file()

        return self

    def update_user_var_from_dict(self, data: dict) -> Device:
        """Parses user variables from Cybro scgi server response.

        Args:
            data: object with the data received from a
                Cybro scgi server request.

        Returns:
            The updated Device object.
        """
        # update user variable
        try:
            for _var in data["var"]:
                if _var == "name":
                    self.vars.update({data["var"]["name"]: Var.from_dict(data)})
                else:
                    self.vars.update({_var["name"]: Var.from_dict(_var)})
        except (KeyError, TypeError):
            pass
        return self

    def update_var(self, data: dict, var_type: VarType = 0) -> str | bool | int | float:
        """Return Value of a single var response.

        Args:
            data: Update the variable object with the data received from a
                Cybro scgi request.
            var_type: Type of read variable (default = str)

        Returns:
            The value of the var
        """
        try:
            for _var in data["var"]:
                if _var == "name":
                    # single var entry found
                    _var = data["var"]
                self.vars.update({_var["name"]: Var.from_dict(_var)})
                self.vars_types.update({_var["name"]: var_type})
                self.user_vars.update({_var["name"]: ""})
                return self.vars[_var["name"]].value
        except (KeyError, TypeError):
            pass
        return "?"

    def add_var(self, name: str, var_type: VarType = 0) -> None:
        """Adds a variable to the update list.

        Args:
            name: Variable name to read eg: c1000.scan_time
            var_type: Optionally defines a Varaibe Type"""
        self.user_vars.update({name: ""})
        self.vars_types.update({name: var_type})

    def remove_var(self, name: str) -> None:
        """Removes a variable from the read list.

        Args:
            name: Variable name to delete from user_vars"""
        self.user_vars.pop(name, "")
        self.vars_types.pop(name, "")


class VarType(IntEnum):
    """Enumeration representing variable types"""

    STR = 0
    INT = 1
    FLOAT = 2
    BOOL = 3



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cybro']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.0',
 'backoff>=1.11.0',
 'cachetools>=5.0.0',
 'xmltodict>=0.13.0,<0.14.0',
 'yarl>=1.7.2']

setup_kwargs = {
    'name': 'cybro',
    'version': '0.0.8',
    'description': 'Asynchronous Python client for Cybro scgi server.',
    'long_description': '# python-cybro\n\n[![GitHub Release][releases-shield]][releases]\n[![GitHub Activity][commits-shield]][commits]\n[![License][license-shield]](LICENSE)\n\n[![pre-commit][pre-commit-shield]][pre-commit]\n[![Black][black-shield]][black]\n[![Code Coverage][codecov-shield]][codecov]\n\n[![Project Maintenance][maintenance-shield]][user_profile]\n\n## Functionality\n\nPython library to communicate with a cybro scgi server\nTo use this library you need to have a running scgi server (it could be a docker container or native installed).\nFurther information of the docker container can be found here: [![dockerhub][scgi-docker-shield]][scgi-docker]\n\n## Tested scgi server\n\n- Cybrotech scgi server v3.1.3\n\n## Contributions are welcome!\n\nIf you want to contribute to this please read the [Contribution guidelines](https://github.com/killer0071234/python-cybro/blob/master/CONTRIBUTING.md)\n\n---\n\n[black]: https://github.com/psf/black\n[black-shield]: https://img.shields.io/badge/code%20style-black-000000.svg?style=for-the-badge\n[commits-shield]: https://img.shields.io/github/commit-activity/y/killer0071234/python-cybro.svg?style=for-the-badge\n[commits]: https://github.com/killer0071234/python-cybro/commits/main\n[codecov-shield]: https://img.shields.io/codecov/c/gh/killer0071234/python-cybro?style=for-the-badge&token=2VFGXXQ4N0\n[codecov]: https://codecov.io/gh/killer0071234/python-cybro\n[pre-commit]: https://github.com/pre-commit/pre-commit\n[pre-commit-shield]: https://img.shields.io/badge/pre--commit-enabled-brightgreen?style=for-the-badge\n[license-shield]: https://img.shields.io/github/license/killer0071234/python-cybro.svg?style=for-the-badge\n[maintenance-shield]: https://img.shields.io/badge/maintainer-@killer0071234-blue.svg?style=for-the-badge\n[releases-shield]: https://img.shields.io/github/release/killer0071234/python-cybro.svg?style=for-the-badge\n[releases]: https://github.com/killer0071234/python-cybro/releases\n[user_profile]: https://github.com/killer0071234\n[scgi-docker-shield]: https://img.shields.io/badge/dockerhub-cybroscgiserver-brightgreen.svg?style=for-the-badge\n[scgi-docker]: https://hub.docker.com/r/killer007/cybroscgiserver\n',
    'author': 'Daniel Gangl',
    'author_email': 'killer007@gmx.at',
    'maintainer': 'Daniel Gangl',
    'maintainer_email': 'killer007@gmx.at',
    'url': 'https://github.com/killer0071234/python-cybro',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
